
	----------------------------------------------------------------------
	-- Leatrix Tunes 8.2.04 (23rd October 2019)
	----------------------------------------------------------------------

	--  Create global table
	_G.LeaTunesDB = _G.LeaTunesDB or {}

	-- Create local tables
	local LeaTunesLC, LeaTunesCB = {}, {}

	-- Version
	LeaTunesLC["AddonVer"] = "8.2.04"
	LeaTunesLC["RestartReq"] = nil

	-- If client restart is required and has not been done, show warning and quit
	if LeaTunesLC["RestartReq"] then
		local metaVer = GetAddOnMetadata("Leatrix_Tunes", "Version")
		if metaVer and metaVer ~= LeaTunesLC["AddonVer"] then
			C_Timer.After(1, function()
				print("NOTICE!|nYou must fully restart your game client before you can use this version of Leatrix Tunes.")
			end)
			return
		end
	end

	-- Create locals
	local void, Leatrix_Tunes = ...
	local L = Leatrix_Tunes.L
	local frameWidth = 625

	----------------------------------------------------------------------
	-- L10: Functions
	----------------------------------------------------------------------

	-- Print text
	function LeaTunesLC:Print(text)
		DEFAULT_CHAT_FRAME:AddMessage(L[text], 1.0, 0.85, 0.0)
	end

	-- Load a numeric variable and set it to default if it's not within a given range
	function LeaTunesLC:LoadVarNum(var, def, valmin, valmax)
		if LeaTunesDB[var] and type(LeaTunesDB[var]) == "number" and LeaTunesDB[var] >= valmin and LeaTunesDB[var] <= valmax then
			LeaTunesLC[var] = LeaTunesDB[var]
		else
			LeaTunesLC[var] = def
			LeaTunesDB[var] = def
		end
	end

	-- Load an anchor point variable and set it to default if the anchor point is invalid
	function LeaTunesLC:LoadVarAnc(var, def)
		if LeaTunesDB[var] and type(LeaTunesDB[var]) == "string" and LeaTunesDB[var] == "CENTER" or LeaTunesDB[var] == "TOP" or LeaTunesDB[var] == "BOTTOM" or LeaTunesDB[var] == "LEFT" or LeaTunesDB[var] == "RIGHT" or LeaTunesDB[var] == "TOPLEFT" or LeaTunesDB[var] == "TOPRIGHT" or LeaTunesDB[var] == "BOTTOMLEFT" or LeaTunesDB[var] == "BOTTOMRIGHT" then
			LeaTunesLC[var] = LeaTunesDB[var]
		else
			LeaTunesLC[var] = def
			LeaTunesDB[var] = def
		end
	end

	-- Load a string variable or set it to default if it's not set to "On" or "Off"
	function LeaTunesLC:LoadVarChk(var, def)
		if LeaTunesDB[var] and type(LeaTunesDB[var]) == "string" and LeaTunesDB[var] == "On" or LeaTunesDB[var] == "Off" then
			LeaTunesLC[var] = LeaTunesDB[var]
		else
			LeaTunesLC[var] = def
			LeaTunesDB[var] = def
		end
	end

	-- Lock and unlock an item
	function LeaTunesLC:LockItem(item, lock)
		if lock then
			item:Disable()
			item:SetAlpha(0.3)
		else
			item:Enable()
			item:SetAlpha(1.0)
		end
	end

	-- Create an editbox
	function LeaTunesLC:CreateEditBox(frame, parent, width, maxchars, anchor, x, y)

		-- Create editbox
		local eb = CreateFrame("EditBox", nil, parent, "InputBoxTemplate")
		eb:SetPoint(anchor, x, y)
		eb:SetWidth(width)
		eb:SetHeight(24)
		eb:SetFontObject("GameFontNormal")
		eb:SetTextColor(1.0, 1.0, 1.0)
		eb:SetAutoFocus(false) 
		eb:SetMaxLetters(maxchars) 
		eb:SetScript("OnEscapePressed", eb.ClearFocus)
		eb:SetScript("OnEnterPressed", eb.ClearFocus)

		-- Add editbox border and backdrop
		eb.f = CreateFrame("FRAME", nil, eb)
		eb.f:SetBackdrop({bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", tile = false, tileSize = 16, edgeSize = 16, insets = { left = 5, right = 5, top = 5, bottom = 5 }})
		eb.f:SetPoint("LEFT", -6, 0)
		eb.f:SetWidth(eb:GetWidth()+6)
		eb.f:SetHeight(eb:GetHeight())
		eb.f:SetBackdropColor(1.0, 1.0, 1.0, 0.3)

		return eb

	end

	-- Create a button
	function LeaTunesLC:CreateButton(name, frame, label, anchor, x, y, height)
		local mbtn = CreateFrame("Button", nil, frame, "UIPanelButtonTemplate")
		LeaTunesCB[name] = mbtn
		mbtn:SetHeight(height)
		mbtn:SetPoint(anchor, x, y)
		mbtn:SetHitRectInsets(0, 0, 0, 0)
		mbtn:SetText(L[label])

		-- Create fontstring and set button width based on it
		mbtn.f = mbtn:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
		mbtn.f:SetText(L[label])
		mbtn:SetWidth(mbtn.f:GetStringWidth() + 20)

		-- Set skinned button textures
		mbtn:SetNormalTexture("Interface\\AddOns\\Leatrix_Tunes\\Leatrix_Tunes.blp")
		mbtn:GetNormalTexture():SetTexCoord(0.5, 1, 0, 1)
		mbtn:SetHighlightTexture("Interface\\AddOns\\Leatrix_Tunes\\Leatrix_Tunes.blp")
		mbtn:GetHighlightTexture():SetTexCoord(0, 0.5, 0, 1)

		-- Hide the default textures
		mbtn:HookScript("OnShow", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnEnable", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnDisable", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnMouseDown", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)
		mbtn:HookScript("OnMouseUp", function() mbtn.Left:Hide(); mbtn.Middle:Hide(); mbtn.Right:Hide() end)

		return mbtn
	end

	-- Create a subheading
	function LeaTunesLC:MakeTx(frame, title, x, y)
		local text = frame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
		text:SetPoint("TOPLEFT", x, y)
		text:SetText(L[title])
		return text
	end

	-- Create a checkbox
	function LeaTunesLC:MakeCB(field, caption, x, y)

		-- Create the checkbox
		local Cbox = CreateFrame("CheckButton", nil, LeaTunesLC["PageF"], "ChatConfigCheckButtonTemplate")
		LeaTunesCB[field] = Cbox
		Cbox:SetPoint("TOPLEFT",x, y)

		-- Add label and tooltip
		Cbox.f = Cbox:CreateFontString(nil, 'ARTWORK', 'GameFontHighlight')
		Cbox.f:SetPoint('LEFT', 20, 0)
		Cbox.f:SetText(L[caption])

		-- Set label parameters
		Cbox.f:SetJustifyH("LEFT")
		Cbox.f:SetWordWrap(false)

		-- Set maximum label width
		if Cbox.f:GetWidth() > 50 then
			Cbox.f:SetWidth(50)
		end

		-- Set checkbox click width
		if Cbox.f:GetStringWidth() > 50 then
			Cbox:SetHitRectInsets(0, -40, 0, 0)
		else
			Cbox:SetHitRectInsets(0, -Cbox.f:GetStringWidth() + 4, 0, 0)
		end

		-- Set default checkbox state
		Cbox:SetScript('OnShow', function(self)
			if LeaTunesLC[field] == "On" then
				self:SetChecked(true)
			else
				self:SetChecked(false)
			end
		end)

		-- Process clicks
		Cbox:SetScript('OnClick', function()
			if Cbox:GetChecked() then
				LeaTunesLC[field] = "On"
			else
				LeaTunesLC[field] = "Off"
			end
		end)
	end

	----------------------------------------------------------------------
	-- L20: Player
	----------------------------------------------------------------------

	function LeaTunesLC:Player()

		-- Create music table
		Leatrix_Tunes["Listing"] = {}

		-- Create panel
		local PageF = CreateFrame("Frame", nil, UIParent)

		-- Make it a system frame
		_G["LeaTunesGlobalPanel"] = PageF
		table.insert(UISpecialFrames, "LeaTunesGlobalPanel")

		-- Set frame parameters
		LeaTunesLC["PageF"] = PageF
		PageF:SetSize(frameWidth, 308)
		PageF:SetScale(1.1)
		PageF:Hide()
		PageF:SetFrameStrata("FULLSCREEN_DIALOG")
		PageF:SetFrameLevel(50)
		PageF:SetClampedToScreen(true)
		PageF:SetClampRectInsets(550, -550, -200, 200)
		PageF:ClearAllPoints()
		PageF:SetPoint("CENTER", 0, 0)
		PageF:EnableMouse(true)
		PageF:SetMovable(true)
		PageF:RegisterForDrag("LeftButton")
		PageF:SetScript("OnDragStart", PageF.StartMoving)
		PageF:SetScript("OnDragStop", function()
			PageF:StopMovingOrSizing()
			PageF:SetUserPlaced(false)
			-- Save panel position
			LeaTunesLC["MainPanelA"], void, LeaTunesLC["MainPanelR"], LeaTunesLC["MainPanelX"], LeaTunesLC["MainPanelY"] = PageF:GetPoint()
		end)

		-- Add background color
		PageF.t = PageF:CreateTexture(nil, "BACKGROUND")
		PageF.t:SetAllPoints()
		PageF.t:SetColorTexture(0.05, 0.05, 0.05, 0.9)

		-- Add textures
		PageF.mainTex = PageF:CreateTexture(nil, "BORDER")
		PageF.mainTex:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
		PageF.mainTex:SetSize(frameWidth, 273)
		PageF.mainTex:SetPoint("TOPRIGHT")
		PageF.mainTex:SetVertexColor(0.7, 0.7, 0.7, 0.7)
		PageF.mainTex:SetTexCoord(0.09, 1, 0, 1)

		PageF.footTex = PageF:CreateTexture(nil, "BORDER")
		PageF.footTex:SetTexture("Interface\\ACHIEVEMENTFRAME\\UI-GuildAchievement-Parchment-Horizontal-Desaturated.png")
		PageF.footTex:SetSize(frameWidth, 48)
		PageF.footTex:SetPoint("BOTTOM")
		PageF.footTex:SetVertexColor(0.5, 0.5, 0.5, 1.0)

		-- Add close Button
		PageF.cb = CreateFrame("Button", nil, PageF, "UIPanelCloseButton") 
		PageF.cb:SetSize(30, 30)
		PageF.cb:SetPoint("TOPRIGHT", 0, 0)

		-- Set panel position when shown
		PageF:SetScript("OnShow", function()
			PageF:ClearAllPoints()
			PageF:SetPoint(LeaTunesLC["MainPanelA"], UIParent, LeaTunesLC["MainPanelR"], LeaTunesLC["MainPanelX"], LeaTunesLC["MainPanelY"])
		end)

		-- Create checkboxes
		LeaTunesLC:MakeCB("SoundMP3", "MP3", 416, -276)
		LeaTunesLC:MakeCB("SoundOGG", "OGG", 486, -276)

		-- Position checkboxes
		LeaTunesCB["SoundOGG"]:ClearAllPoints()
		LeaTunesCB["SoundOGG"]:SetPoint("BOTTOMRIGHT", -50, 10)
		LeaTunesCB["SoundMP3"]:ClearAllPoints()
		LeaTunesCB["SoundMP3"]:SetPoint("RIGHT", LeaTunesCB["SoundOGG"], "LEFT", -50, 0)

		-- Create locals
		local ListData, searchTable = {}, {}
		local scrollFrame, musicHandle, lastPlayed, void
		local numButtons = 15

		-- Show list items
		local function UpdateList()
			-- Add headers to ListData if needed
			if not strfind(ListData[1], "|c") then 
				tinsert(ListData, 1, "|cffffd800" .. L["Leatrix Tunes"] .. " " .. LeaTunesLC["AddonVer"])
				tinsert(ListData, 2, "|cffffffaa{" .. #Leatrix_Tunes["Listing"] - 1 .. " " .. L["results"] .. "}")
				tinsert(ListData, 3, "|cffffffff")
				tinsert(ListData, 4, "|cffffd800" .. L["Sound Files"])
			end
			-- Update buttons
			FauxScrollFrame_Update(scrollFrame, #ListData, numButtons, 16)
			for index = 1, numButtons do
				local offset = index + FauxScrollFrame_GetOffset(scrollFrame)
				local button = scrollFrame.buttons[index]
				button.index = offset
				if offset <= #ListData then
					-- Show track listing
					button:SetText(ListData[offset])
					-- Set width of highlight texture
					if button:GetTextWidth() > frameWidth - 60 then
						button.t:SetSize(frameWidth - 60, 16)
					else
						button.t:SetSize(button:GetTextWidth(), 16)
					end
					-- Show the button
					button:Show()
					-- Hide highlight bar texture by default
					button.s:Hide()
					-- Hide highlight bar if the button is a heading
					if strfind(button:GetText(), "|c") then button.t:Hide() end
					-- Show last played track highlight bar texture
					if lastPlayed == button:GetText() then
						button.s:Show()
					end
					-- Set width of highlight bar
					if button:GetTextWidth() > frameWidth - 60 then
						button.s:SetSize(frameWidth - 60, 16)
					else
						button.s:SetSize(button:GetTextWidth(), 16)
					end
					-- Limit click to label width
					local bWidth = button:GetFontString():GetStringWidth() or 0
					if bWidth > frameWidth - 60 then bWidth = frameWidth - 60 end
					button:SetHitRectInsets(0, 454 - bWidth, 0, 0)
					-- Disable label click movement
					button:SetPushedTextOffset(0, 0)
					-- Disable word wrap and set width
					button:GetFontString():SetWidth(frameWidth - 60)
					button:GetFontString():SetWordWrap(false)
				else
					button:Hide()
				end
			end
		end

		-- Create scroll bar
		scrollFrame = CreateFrame("ScrollFrame", "LeaTunesScrollFrame", LeaTunesLC["PageF"], "FauxScrollFrameTemplate")
		scrollFrame:SetPoint("TOPLEFT", 0, -32)
		scrollFrame:SetPoint("BOTTOMRIGHT", -30, 52)
		scrollFrame:SetScript("OnVerticalScroll", function(self, offset)
			FauxScrollFrame_OnVerticalScroll(self, offset, 16, UpdateList)
		end)

		-- Add stop button
		local stopBtn = LeaTunesLC:CreateButton("StopPlaybackButton", LeaTunesLC["PageF"], "Stop", "BOTTOMRIGHT", -16, 12, 25)
		stopBtn:Hide(); stopBtn:Show()
		LeaTunesLC:LockItem(stopBtn, true)
		stopBtn:SetScript("OnClick", function()
			-- Stop currently playing track
			if musicHandle then
				StopSound(musicHandle)
				musicHandle = nil
			end
			-- Hide highlight bars
			lastPlayed = ""
			UpdateList()
			-- Lock button
			LeaTunesLC:LockItem(stopBtn, true)
		end)

		-- Create editbox for search
		local searchLabel = LeaTunesLC:MakeTx(LeaTunesLC["PageF"], "Search", 16, -278)
		searchLabel:ClearAllPoints()
		searchLabel:SetPoint("BOTTOMLEFT", 16, 17)

		local sBox = LeaTunesLC:CreateEditBox("SearchBox", LeaTunesLC["PageF"], 300, 10, "TOPLEFT", 101, -272)
		LeaTunesCB["sBox"] = sBox
		sBox:SetMaxLetters(70)
		sBox:ClearAllPoints()
		sBox:SetPoint("LEFT", searchLabel, "RIGHT", 16, 0)

		-- Reposition stop button so its next to the search box
		stopBtn:ClearAllPoints()
		stopBtn:SetPoint("LEFT", sBox, "RIGHT", 10, 0)

		-- Function to show search results
		local function ShowSearchResults()

			-- Get unescaped editbox text
			local searchText = gsub(strlower(sBox:GetText()), '(['..("%^$().[]*+-?"):gsub("(.)", "%%%1")..'])', "%%%1")
			local trackCount = #Leatrix_Tunes["Listing"]

			-- Wipe the track listing
			wipe(ListData)

			-- Set headings
			ListData[1] = "|cffffd800" .. L["Leatrix Tunes"] .. " " .. LeaTunesLC["AddonVer"]
			if searchText == "" then
				ListData = Leatrix_Tunes["Listing"]
				UpdateList()
			else
				ListData[2] = ""
			end
			ListData[3] = "|cffffffff"
			ListData[4] = "|cffffd800" .. L["Sound Files"]

			-- Traverse music listing and populate ListData
			if searchText ~= "" then
				local word1, word2, word3, word4, word5 = strsplit(" ", (strtrim(searchText):gsub("%s+", " ")))
				for k = 1, trackCount do
					local v = strlower(Leatrix_Tunes["Listing"][k])
					if strfind(v, "#") and strfind(v, word1) then
						if not word2 or word2 ~= "" and strfind(v, word2) then
							if not word3 or word3 ~= "" and strfind(v, word3) then
								if not word4 or word4 ~= "" and strfind(v, word4) then
									if not word5 or word5 ~= "" and strfind(v, word5) then
										-- Show track
										tinsert(ListData, Leatrix_Tunes["Listing"][k])
									end
								end
							end
						end
					end
				end
				-- Set results tag
				if #ListData == 5 then
					ListData[2] = "|cffffffaa{" .. #ListData - 4 .. " " .. L["result"] .. "}"
				else
					ListData[2] = "|cffffffaa{" .. #ListData - 4 .. " " .. L["results"] .. "}"
				end
			end
			-- Show message if no results found
			if #ListData == 4 then
				ListData[4] = "|cffffd800" .. L["No search results found"]
				ListData[5] = "|cffffffff" .. L["So I guess there's nothing to see here."]
			end
			-- Show message if no sound categories are checked
			if LeaTunesLC["SoundMP3"] == "Off" and LeaTunesLC["SoundOGG"] == "Off" then
				ListData[4] = "|cffffd800" .. L["Oops!"]
				ListData[5] = "|cffffffff" .. L["You need to select at least one sound category."]
			end
			-- Refresh the track listing
			UpdateList()
			-- Set track listing to top
			scrollFrame:SetVerticalScroll(0)
		end

		-- When editbox is changed by user, show search results
		sBox:HookScript("OnTextChanged", function(self, userInput)
			if userInput then
				if LeaTunesLC.Timer then LeaTunesLC.Timer:Cancel() end
				LeaTunesLC.Timer = C_Timer.NewTimer(0.5, function()
					ListData = searchTable
					ShowSearchResults()
				end)
			end
		end)

		-- When enter key is pressed, show search results
		sBox:HookScript("OnEnterPressed", function()
			ListData = searchTable
			ShowSearchResults()
		end)

		-- Function to show sound listing
		local function SetListingFunc()
			wipe(Leatrix_Tunes["Listing"])
			-- Populate sound table
			if LeaTunesLC["SoundMP3"] == "On" then
				for i = 1, #Leatrix_Tunes["MP3"] do
					tinsert(Leatrix_Tunes["Listing"], Leatrix_Tunes["MP3"][i])
				end
			end
			if LeaTunesLC["SoundOGG"] == "On" then
				for i = 1, #Leatrix_Tunes["OGG"] do
					tinsert(Leatrix_Tunes["Listing"], Leatrix_Tunes["OGG"][i])
				end
			end
			-- Sort the table
			table.sort(Leatrix_Tunes["Listing"])
			-- Show headings if needed
			if LeaTunesLC["SoundMP3"] == "Off" and LeaTunesLC["SoundOGG"] == "Off" then
				tinsert(Leatrix_Tunes["Listing"], "|cffffd800" .. L["Leatrix Tunes"] .. " " .. LeaTunesLC["AddonVer"])
				tinsert(Leatrix_Tunes["Listing"], "|cffffffaa{" .. #Leatrix_Tunes["Listing"] - 1 .. " " .. L["results"] .. "}")
				tinsert(Leatrix_Tunes["Listing"], "|cffffffff")
				tinsert(Leatrix_Tunes["Listing"], "|cffffd800" .. L["Oops!"])
				tinsert(Leatrix_Tunes["Listing"], "|cffffffff" .. L["You need to select at least one sound category."])
			end
			-- If editbox is not empty, show search results
			if LeaTunesCB["sBox"]:GetText() ~= "" then
				ListData = searchTable
				ShowSearchResults()
			end
			-- Clear editbox focus
			sBox:ClearFocus()
			-- Update listing buttons
			UpdateList()
		end

		LeaTunesCB["SoundMP3"]:HookScript("OnClick", function() C_Timer.After(0.001, SetListingFunc) end)
		LeaTunesCB["SoundOGG"]:HookScript("OnClick", function() C_Timer.After(0.001, SetListingFunc) end)

		-- Create list items
		scrollFrame.buttons = {}
		for i = 1, numButtons do
			scrollFrame.buttons[i] = CreateFrame("Button", nil, LeaTunesLC["PageF"])
			local button = scrollFrame.buttons[i]

			button:SetSize(470 - 14, 16)
			button:SetNormalFontObject("GameFontHighlightLeft")
			button:SetPoint("TOPLEFT", 16, -6+ -(i - 1) * 16 - 8)

			-- Create highlight bar texture
			button.t = button:CreateTexture(nil, "BACKGROUND")
			button.t:SetPoint("TOPLEFT", button, 0, 0)
			button.t:SetSize(516, 16)

			button.t:SetColorTexture(0.3, 0.3, 0.0, 0.8)
			button.t:SetAlpha(0.7)
			button.t:Hide()

			-- Create last playing highlight bar texture
			button.s = button:CreateTexture(nil, "BACKGROUND")
			button.s:SetPoint("TOPLEFT", button, 0, 0)
			button.s:SetSize(516, 16)

			button.s:SetColorTexture(0.3, 0.4, 0.00, 0.6)
			button.s:Hide()

			button:SetScript("OnEnter", function()
				-- Highlight links only
				if not string.match(button:GetText() or "", "|c") then
					button.t:Show()
				end
			end)

			button:SetScript("OnLeave", function()
				button.t:Hide()
			end)

			button:RegisterForClicks("LeftButtonUp", "RightButtonUp")

			-- Click handler for track
			button:SetScript("OnClick", function(self, btn)
				if btn == "LeftButton" then
					-- Remove focus from search box
					sBox:ClearFocus()
					-- Get clicked track text
					local item = self:GetText()
					-- Do nothing if its a blank line or informational heading
					if not item or strfind(item, "|c") then return end
					if strfind(item, "#") then
						-- Print track name in chat if shift is held
						if IsShiftKeyDown() then
							DEFAULT_CHAT_FRAME:AddMessage(item)
							return
						end
						-- Enable sound if required
						if GetCVar("Sound_EnableAllSound") == "0" then SetCVar("Sound_EnableAllSound", "1") end
						-- Disable music if it's currently enabled
						if GetCVar("Sound_EnableMusic") == "1" then	SetCVar("Sound_EnableMusic", "0") end
						-- Enable the stop button
						LeaTunesLC:LockItem(stopBtn, false)
						-- Store the track we are about to play
						lastPlayed = item
						-- Stop currently playing track if there is one
						if musicHandle then StopSound(musicHandle) end
						-- Play track
						local file, soundID = item:match("([^,]+)%#([^,]+)")
						void, musicHandle = PlaySoundFile(soundID, "Master", false, true)
						-- Show static highlight bar
						for index = 1, numButtons do
							local button = scrollFrame.buttons[index]
							local scanItem = button:GetText()
							if scanItem and scanItem == item then
								button.s:Show()
							else
								button.s:Hide()
							end
						end
					end
				elseif btn == "RightButton" and IsShiftKeyDown() then
					-- Remove focus from search box
					sBox:ClearFocus()
					-- Get clicked track text
					local item = self:GetText()
					-- Do nothing if its a blank line or informational heading
					if not item or strfind(item, "|c") then return end
					if strfind(item, "#") then
						-- Print track name in chat editbox and highlight it
						local eBox = ChatEdit_ChooseBoxForSend()
						ChatEdit_ActivateChat(eBox)
						eBox:Insert(item)
						eBox:HighlightText()
					end
				end
			end)
		end

		-- Delete the global scroll frame pointer
		_G.LeaTunesScrollFrame = nil

		-- Manage events
		LeaTunesLC["PageF"]:RegisterEvent("PLAYER_LOGOUT")
		LeaTunesLC["PageF"]:RegisterEvent("UI_SCALE_CHANGED")
		LeaTunesLC["PageF"]:SetScript("OnEvent", function(self, event)
			if event == "PLAYER_LOGOUT" then
				-- Stop playing at reload or logout
				if musicHandle then
					StopSound(musicHandle)
				end
			elseif event == "UI_SCALE_CHANGED" then
				-- Refresh list
				UpdateList()
			end
		end)

		-- Show sound listing on startup
		ListData = Leatrix_Tunes["Listing"]
		SetListingFunc()

		-- Release memory
		LeaTunesLC.Player = nil

	end

	----------------------------------------------------------------------
	-- L30: Events
	----------------------------------------------------------------------

	-- Create event frame
	local eventFrame = CreateFrame("FRAME")
	eventFrame:RegisterEvent("ADDON_LOADED")
	eventFrame:RegisterEvent("PLAYER_LOGIN")
	eventFrame:RegisterEvent("PLAYER_LOGOUT")
	eventFrame:SetScript("OnEvent", function(self, event, arg1)

		if event == "ADDON_LOADED" and arg1 == "Leatrix_Tunes" then
			-- Load settings or set defaults
			LeaTunesLC:LoadVarAnc("MainPanelA", "CENTER")
			LeaTunesLC:LoadVarAnc("MainPanelR", "CENTER")
			LeaTunesLC:LoadVarNum("MainPanelX", 0, -5000, 5000)
			LeaTunesLC:LoadVarNum("MainPanelY", 0, -5000, 5000)
			LeaTunesLC:LoadVarChk("SoundMP3", "On")
			LeaTunesLC:LoadVarChk("SoundOGG", "On")

		elseif event == "PLAYER_LOGIN" then
			-- Run main function
			LeaTunesLC:Player()
			collectgarbage()

		elseif event == "PLAYER_LOGOUT" then
			-- Save settings
			LeaTunesDB["MainPanelA"] = LeaTunesLC["MainPanelA"]
			LeaTunesDB["MainPanelR"] = LeaTunesLC["MainPanelR"]
			LeaTunesDB["MainPanelX"] = LeaTunesLC["MainPanelX"]
			LeaTunesDB["MainPanelY"] = LeaTunesLC["MainPanelY"]
			LeaTunesDB["SoundMP3"] = LeaTunesLC["SoundMP3"]
			LeaTunesDB["SoundOGG"] = LeaTunesLC["SoundOGG"]
		end

	end)

	----------------------------------------------------------------------
	-- L40: Commands
	----------------------------------------------------------------------

	-- Slash command function
	local function SlashFunc(str)
		local str = string.lower(str)
		if str and str ~= "" then
			-- Traverse parameters
			if str == "ver" then
				-- Print addon version
				LeaTunesLC:Print("Leatrix Tunes " .. LeaTunesLC["AddonVer"])
				return
			elseif str == "rsnd" then
				-- Restart the sound system
				if LeaTunesCB["StopPlaybackButton"] then LeaTunesCB["StopPlaybackButton"]:Click() end 
				Sound_GameSystem_RestartSoundSystem()
				LeaTunesLC:Print("Sound system restarted.")
				return
			elseif str == "reset" then
				-- Reset layout
				LeaTunesLC["MainPanelA"], LeaTunesLC["MainPanelR"], LeaTunesLC["MainPanelX"], LeaTunesLC["MainPanelY"] = "CENTER", "CENTER", 0, 0
				LeaTunesLC["PageF"]:ClearAllPoints()
				LeaTunesLC["PageF"]:SetPoint(LeaTunesLC["MainPanelA"], UIParent, LeaTunesLC["MainPanelR"], LeaTunesLC["MainPanelX"], LeaTunesLC["MainPanelY"])
				return
			elseif str == "wipe" then
				-- Wipe settings
				wipe(LeaTunesDB)
				eventFrame:UnregisterAllEvents()
				ReloadUI()
			else
				-- Invalid command entered
				LeaTunesLC:Print("Invalid command.")
				return
			end
		else
			-- Prevent panel from showing if a game options panel is showing
			if InterfaceOptionsFrame:IsShown() or VideoOptionsFrame:IsShown() or ChatConfigFrame:IsShown() then return end
			-- Prevent panel from showing if Blizzard Store is showing
			if StoreFrame and StoreFrame:GetAttribute("isshown") then return end
			-- Toggle the main panel
			if LeaTunesLC["PageF"]:IsShown() then
				LeaTunesLC["PageF"]:Hide()
			else
				LeaTunesLC["PageF"]:Show()
			end
		end
	end

	-- Add slash commands
	_G.SLASH_Leatrix_Tunes1 = "/ltt"
	_G.SLASH_Leatrix_Tunes2 = "/leatunes" 
	SlashCmdList["Leatrix_Tunes"] = function(self)
		-- Run slash command function
		SlashFunc(self)
		-- Redirect tainted variables
		RunScript('ACTIVE_CHAT_EDIT_BOX = ACTIVE_CHAT_EDIT_BOX')
		RunScript('LAST_ACTIVE_CHAT_EDIT_BOX = LAST_ACTIVE_CHAT_EDIT_BOX')
	end

	----------------------------------------------------------------------
	-- Create panel in game options panel
	----------------------------------------------------------------------

	do

		local interPanel = CreateFrame("FRAME")
		interPanel.name = "Leatrix Tunes"

		local maintitle = LeaTunesLC:MakeTx(interPanel, "Leatrix Tunes", 0, 0)
		maintitle:SetFont(maintitle:GetFont(), 72)
		maintitle:ClearAllPoints()
		maintitle:SetPoint("TOP", 0, -72)

		local subTitle = LeaTunesLC:MakeTx(interPanel, "curseforge.com/wow/addons/leatrix-tunes", 0, 0)
		subTitle:SetFont(subTitle:GetFont(), 20)
		subTitle:ClearAllPoints()
		subTitle:SetPoint("BOTTOM", 0, 72)

		local slashTitle = LeaTunesLC:MakeTx(interPanel, "/ltt", 0, 0)
		slashTitle:SetFont(slashTitle:GetFont(), 72)
		slashTitle:ClearAllPoints()
		slashTitle:SetPoint("BOTTOM", subTitle, "TOP", 0, 40)

		local pTex = interPanel:CreateTexture(nil, "BACKGROUND")
		pTex:SetAllPoints()
		pTex:SetTexture("Interface\\GLUES\\Models\\UI_MainMenu\\swordgradient2")
		pTex:SetAlpha(0.2)
		pTex:SetTexCoord(0, 1, 1, 0)

		InterfaceOptions_AddCategory(interPanel)

	end
